@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2008 - 2010 Red Hat, Inc.
+ * Copyright (C) 2008 - 2011 Red Hat, Inc.
  * Copyright (C) 2008 Novell, Inc.
  */
 
@@ -169,6 +169,8 @@ nm_dhcp6_config_class_init (NMDHCP6ConfigClass *config_class)
 	 * NMDHCP6Config:options:
 	 *
 	 * The #GHashTable containing options of the configuration.
+	 *
+	 * Type: GHashTable<utf8,GValue>
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_OPTIONS,
@@ -203,7 +205,7 @@ nm_dhcp6_config_new (DBusGConnection *connection, const char *object_path)
  *
  * Gets all the options contained in the configuration.
  *
- * Returns: the #GHashTable containing strings for keys and values.
+ * Returns: (transfer none) (element-type utf8 GObject.Value): the #GHashTable containing strings for keys and values.
  * This is the internal copy used by the configuration, and must not be modified.
  **/
 GHashTable *
@@ -218,7 +220,8 @@ nm_dhcp6_config_get_options (NMDHCP6Config *config)
 	if (!_nm_object_get_property (NM_OBJECT (config),
 	                              NM_DBUS_INTERFACE_DHCP6_CONFIG,
 	                              "Options",
-	                              &value))
+	                              &value,
+	                              NULL))
 		goto out;
 
 	demarshal_dhcp6_options (NM_OBJECT (config), NULL, &value, &priv->options);	
