@@ -39,23 +39,6 @@ G_BEGIN_DECLS
 #define NM_IS_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), NM_TYPE_CONNECTION))
 #define NM_CONNECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_CONNECTION, NMConnectionClass))
 
-/**
- * NMConnectionScope:
- * @NM_CONNECTION_SCOPE_UNKNOWN: scope not known or not yet set
- * @NM_CONNECTION_SCOPE_SYSTEM: connection is provided by the system settings
- *   service
- * @NM_CONNECTION_SCOPE_USER: connection is provided by a user settings service
- *
- * Connection scope indicated what settings service, if any, provides the
- * connection.
- *
- **/
-typedef enum {
-	NM_CONNECTION_SCOPE_UNKNOWN = 0,
-	NM_CONNECTION_SCOPE_SYSTEM,
-	NM_CONNECTION_SCOPE_USER
-} NMConnectionScope;
-
 
 /**
  * NMConnectionError:
@@ -79,7 +62,6 @@ GType nm_connection_error_get_type (void);
 #define NM_CONNECTION_ERROR nm_connection_error_quark ()
 GQuark nm_connection_error_quark (void);
 
-#define NM_CONNECTION_SCOPE "scope"
 #define NM_CONNECTION_PATH "path"
 
 /**
@@ -107,8 +89,10 @@ NMConnection *nm_connection_new_from_hash (GHashTable *hash, GError **error);
 
 NMConnection *nm_connection_duplicate     (NMConnection *connection);
 
+NMSetting    *nm_connection_create_setting (const char *name);
+
 void          nm_connection_add_setting   (NMConnection *connection,
-								   NMSetting    *setting);
+                                           NMSetting    *setting);
 
 void          nm_connection_remove_setting (NMConnection *connection,
                                             GType         setting_type);
@@ -117,7 +101,7 @@ NMSetting    *nm_connection_get_setting   (NMConnection *connection,
                                            GType         setting_type);
 
 NMSetting    *nm_connection_get_setting_by_name (NMConnection *connection,
-									    const char *name);
+                                                 const char   *name);
 
 gboolean      nm_connection_replace_settings (NMConnection *connection,
                                               GHashTable *new_settings,
@@ -141,31 +125,31 @@ void          nm_connection_clear_secrets (NMConnection *connection);
 
 gboolean      nm_connection_update_secrets (NMConnection *connection,
                                             const char *setting_name,
-                                            GHashTable *setting_secrets,
+                                            GHashTable *all_secrets,
                                             GError **error);
 
-void             nm_connection_set_scope (NMConnection *connection,
-                                                 NMConnectionScope scope);
+void          nm_connection_set_path      (NMConnection *connection,
+                                           const char *path);
 
-NMConnectionScope nm_connection_get_scope (NMConnection *connection);
-
-void             nm_connection_set_path (NMConnection *connection,
-                                         const char *path);
-
-const char *     nm_connection_get_path (NMConnection *connection);
+const char *  nm_connection_get_path      (NMConnection *connection);
 
 void          nm_connection_for_each_setting_value (NMConnection *connection,
-										  NMSettingValueIterFn func,
-										  gpointer user_data);
+                                                    NMSettingValueIterFn func,
+                                                    gpointer user_data);
+
+GHashTable   *nm_connection_to_hash       (NMConnection *connection,
+                                           NMSettingHashFlags flags);
 
-GHashTable   *nm_connection_to_hash       (NMConnection *connection);
 void          nm_connection_dump          (NMConnection *connection);
 
-NMSetting    *nm_connection_create_setting (const char *name);
+GType         nm_connection_lookup_setting_type (const char *name);
 
-GType nm_connection_lookup_setting_type (const char *name);
+GType         nm_connection_lookup_setting_type_by_quark (GQuark error_quark);
 
-GType nm_connection_lookup_setting_type_by_quark (GQuark error_quark);
+/* Helpers */
+const char *  nm_connection_get_uuid      (NMConnection *connection);
+
+const char *  nm_connection_get_id        (NMConnection *connection);
 
 G_END_DECLS
 
